// Smoothness: 250 is probably good for printing, reduce while working on it to speed things up
$fn=80;

// speaker params
// Seas H1600-08 FU10RB - Full Range - Prestige Series
speaker_external_diameter=100;
speaker_basket_diameter=81;
speaker_magnet_diameter=60;
speaker_internal_depth=43.6;
speaker_external_depth=5.4;
speaker_mounting_hole_diameter=5;
speaker_mounting_hole_offset=89/2;
speaker_number_holes=4;
speaker_hole_angle=45;

// do we have a tweeter?
tweeter = true;

// tweeter enclosure params
// Vifa OX20SC00-04
tweeter_diameter = 31;
tweeter_mounting_diameter = 36;
tweeter_mounting_depth = 7;
tweeter_enclosure_wall = 4;
tweeter_sphere_scale = 2.5;

// enclosure params
wall_thickness=6;
baffle_thickness=12;
enclosure_diameter=150;
screw_hole_diameter=3;
front_trim = 3;
bottom_diameter = 50;
bottom_thickness = 8;
bottom_angle = -10;

// binding posts
post_hole_diameter = 5;
post_vertical_angle = 30;
post_inset = 0.5;
post_inset_diameter = 12;
post_horizontal_angle = 15;

// calculated params
speaker_offset = sqrt(pow(enclosure_diameter/2,2)-pow(speaker_external_diameter/2,2));
speaker_mounting_offset = speaker_offset - speaker_external_depth - front_trim;
sphere_size_at_speaker_mounting_offset = 2*sqrt(pow(enclosure_diameter/2,2)-pow(speaker_mounting_offset,2));

sphere_size_at_back_of_baffle = 2*sqrt(pow(enclosure_diameter/2,2)-pow(speaker_mounting_offset-baffle_thickness,2));

bottom_offset = sqrt(pow(enclosure_diameter/2,2)-pow(bottom_diameter/2,2));
bottom_internal_offset = bottom_offset - bottom_thickness;
bottom_internal_diameter = 2*sqrt(pow((enclosure_diameter-wall_thickness)/2,2)-pow(bottom_internal_offset,2));

tweeter_mount_depth_prescale = tweeter_mounting_depth/tweeter_sphere_scale;
tweeter_pod_internal_radius_at_front = (tweeter_mounting_diameter/2);
tweeter_pod_internal_diameter = 2 * sqrt(pow(tweeter_pod_internal_radius_at_front,2)+pow(tweeter_mount_depth_prescale,2));
tweeter_pod_external_diameter = tweeter_pod_internal_diameter + (2 * tweeter_enclosure_wall);

module tweeterpod(){
  difference(){
    scale([tweeter_sphere_scale,1,1]){        
      sphere(d=tweeter_pod_external_diameter);
    }
    tweeterpodinternals();
    translate([(tweeter_pod_external_diameter/2)+tweeter_mounting_depth,0,0]){
      cube([tweeter_pod_external_diameter,tweeter_pod_external_diameter,tweeter_pod_external_diameter], center=true);
    }
  }
}

module tweeterpodinternals(){
  intersection(){
    scale([tweeter_sphere_scale,1,1]){
      sphere(d=tweeter_pod_internal_diameter);
    }
    union(){
      translate([(tweeter_mounting_depth+2)/2,0,0]){
        rotate([0,90,0]){
          cylinder(d=tweeter_mounting_diameter, h=tweeter_mounting_depth+2, center=true);
        }
      }
      translate([1-((tweeter_pod_internal_diameter/2)*tweeter_sphere_scale)/2,0,0]){
        rotate([0,90,0]){
          cylinder(d=tweeter_diameter, h=2+((tweeter_pod_internal_diameter/2)*tweeter_sphere_scale), center=true);
        }
      }      
    }
  }
  // wire hole
  translate([4-(tweeter_pod_internal_diameter/2)*tweeter_sphere_scale,0,enclosure_diameter/4]){
      cylinder(d=4,h=enclosure_diameter/2, center=true);
  }
}

module enclosure(){
  
  difference(){
    union(){
      // enclosure
      difference(){
        sphere(d=enclosure_diameter);
        sphere(d=enclosure_diameter-(2*wall_thickness));
      }
      //baffle
      translate([speaker_mounting_offset,0,0]){
        rotate([0,-90,0]){
          cylinder(d1=sphere_size_at_speaker_mounting_offset-1, d2=sphere_size_at_back_of_baffle-1, h=baffle_thickness);
        }
      }
      // bottom
      rotate([0,bottom_angle,0]){
        translate([0,0,bottom_internal_offset]){
          cylinder(d1=bottom_internal_diameter, d2=bottom_diameter, h=bottom_thickness);
        }
      }
      
      // tweeter pod
      if(tweeter){
        translate([speaker_offset-front_trim-tweeter_mounting_depth,0,-(( speaker_external_diameter+tweeter_pod_external_diameter)/2)]){        
          tweeterpod();
        }
      }
    }
    
    // slice off face for speaker mounting 

    translate([(enclosure_diameter/2)+speaker_offset,0,0]){
      cube([enclosure_diameter, enclosure_diameter, enclosure_diameter], center=true);
    }

    // take out cylinder for speaker basket
    rotate([0,90,0]){
      cylinder(d=speaker_basket_diameter, h=enclosure_diameter/2);
    }
    
    // cut out mounting hole
    translate([speaker_mounting_offset,0,0]){
      rotate([0,90,0]){
        cylinder(d=speaker_external_diameter, h=enclosure_diameter/2);
      }    
    }
    
    // screw holes in baffle
    translate([speaker_mounting_offset,0,0]){
      rotate([0,-90,0]){
        hole_cutter(speaker_number_holes, speaker_hole_angle, speaker_mounting_hole_offset, screw_hole_diameter, baffle_thickness);
      }
    }
    
    // trim the front to get ring around the driver
    translate([(enclosure_diameter/2)+speaker_offset-front_trim,0,0]){
      cube([enclosure_diameter, enclosure_diameter*2, enclosure_diameter*2], center=true);
    }    
    
    // flatten the bottom
    rotate([0,bottom_angle,0]){
      translate([0,0,(enclosure_diameter/2)+bottom_offset]){
        cube([enclosure_diameter, enclosure_diameter, enclosure_diameter], center=true);
      }
    }
    
    if(tweeter){
      translate([speaker_offset-front_trim-tweeter_mounting_depth,0,-(( speaker_external_diameter+tweeter_pod_external_diameter)/2)]){        
        tweeterpodinternals();
      }
    }    
    
    // binding post holes
    for(i=[-post_horizontal_angle/2:post_horizontal_angle:post_horizontal_angle/2]){
      rotate([0,post_vertical_angle,i]){
        translate([1+wall_thickness-(enclosure_diameter/2),0,0]){
          rotate([0,-90,0]){
            cylinder(d=post_hole_diameter, h=wall_thickness+2);
            cylinder(d=post_inset_diameter, h=post_inset+1);
            translate([0,0,wall_thickness+1-post_inset]){
              cylinder(d=post_inset_diameter, h=post_inset+1);  
            }
          }    
        }    
      }
    }
    
  }
}

module wall_mount(){
  head_diameter = 9;
  slot_width = 4;
  slot_depth = 5;
  slot_length = 15;
  mounting_min_wall = 3;
  cut_depth = slot_depth + mounting_min_wall;
  mounting_depth = slot_depth+(2*mounting_min_wall);
  
  translate([mounting_min_wall-(enclosure_diameter/2)-mounting_depth,0,0]){
    rotate([0,90,0]){
      difference(){
        cylinder(d=slot_length*2,h=mounting_depth);
        translate([-slot_length/2, 0,0]){
          cylinder(d=head_diameter, h=cut_depth);
        }
        // external slot
        translate([-slot_length/2, -slot_width/2, 0]){
          cube([slot_length, slot_width, cut_depth]);
        }
        translate([slot_length/2, 0,0]){
          cylinder(d=slot_width, h=cut_depth);
        }
        
        // internal slot
        translate([-slot_length/2, -head_diameter/2, mounting_min_wall]){
          cube([slot_length, head_diameter, slot_depth]);
        }
        translate([slot_length/2, 0,mounting_min_wall]){
          cylinder(d=head_diameter, h=slot_depth);
        }    
        
      }
    }
  }
}

module driver(){
  speaker_cone_depth_to_diameter_ratio=0.4;
  translate([speaker_mounting_offset+speaker_external_depth,0,0]){
    rotate([0,-90,0]){
      difference(){
        union(){
          cylinder(d=speaker_external_diameter, h=speaker_external_depth);
          translate([0,0,speaker_external_depth]){
            cylinder(d1=speaker_basket_diameter, d2=speaker_magnet_diameter, h=speaker_internal_depth);
          }
        }
        // mounting holes
        hole_cutter(speaker_number_holes, speaker_hole_angle, speaker_mounting_hole_offset, speaker_mounting_hole_diameter, speaker_external_depth+2);        
        // cut out a speheroid cone
        scale([1,1,speaker_cone_depth_to_diameter_ratio]){
          sphere(d=speaker_basket_diameter);
        }
      }
      // dust cap
      translate([0,0,(speaker_basket_diameter*speaker_cone_depth_to_diameter_ratio)/2]){
        scale([1,1,speaker_cone_depth_to_diameter_ratio]){
          sphere(d=speaker_basket_diameter/3);
        }
      }
    }
  }
}

module hole_cutter(number_of_holes, start_angle, centre_offset, hole_diameter, hole_depth){
  for(i=[start_angle:(360/number_of_holes):360+start_angle]){
    rotate([0,0,i]){
      translate([-centre_offset,0,-1]){
        cylinder(d=hole_diameter, h=hole_depth+2);
      }
    }
  }  
}

